//#include "stdio.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h> 
#include <unistd.h> 
#include <errno.h>

#include "SerialPort.h"

using namespace Upp;

SerialPort::SerialPort()
: m_fileDesc(-1)
{
}

void SerialPort::Open( const char * const device, const SerialPortSpeeds sp )
{
	if (m_fileDesc != -1) Close();

	m_fileDesc = ::open(device, O_RDWR, 0644);
	if (m_fileDesc == -1)
	{
		SerialException errExc("Serial device initialization error for device : ");
		errExc << m_deviceName;
		errExc << "\nError: " << strerror(errno);
		throw( errExc );
	}
	else
	{
		struct termios options;

		// Get the current options for the port...
		tcgetattr(m_fileDesc, &options);

		// Set the baud rates
		cfsetispeed(&options, sp);
		cfsetospeed(&options, sp);
		
		// Enable the receiver and set local mode...
		options.c_cflag |= (CLOCAL | CREAD);
		
		// Disable special caracters processing and set RAW mode
		cfmakeraw(&options);
		
		// Set the new options for the port...
		tcsetattr(m_fileDesc, TCSANOW, &options);
	}
}

void SerialPort::Close()
{
	if (m_fileDesc != -1)
	{
		::close(m_fileDesc);
		m_fileDesc = -1;
	}
}


SerialPort::~SerialPort()
{
	Close();
}


String SerialPort::BufToString(const char * const buf, const int nbBytes, const char * const prefixString)
{
	String res;
	res << "\n" << prefixString << "   0x";
	
	for(int c=0; c < nbBytes; c++)
	{
		res << FormatIntHex(buf[c]) << "  "; 
	}
	return res;
}

SerialPort::WriteReturnValues SerialPort::write( int nbBytesToWrite, const char* const buf )
{
	int l_nbBytesWritten= 0;
	
//	displayBuf( buf, nbBytesToWrite, "TX ");
	l_nbBytesWritten = ::write(m_fileDesc, buf, nbBytesToWrite);
	if (l_nbBytesWritten != nbBytesToWrite)
	{
		SerialException errMsg("Write error on serial device : ");
		errMsg << m_deviceName;
		throw( errMsg );
	}
	return WRITE_OK;
}


SerialPort::ReadReturnValues SerialPort::read(char* buf, int& nbBytesToRead, int timeOutus )
{
	int 	l_error		= 0;
	int 	l_nbBytesRead	= 0;
	
	timeval l_TimeOut;
	l_TimeOut.tv_sec=0;
	l_TimeOut.tv_usec = timeOutus;
	
	fd_set 	readFdSet;
	
	// INIT du readFdSet pour utilisation par le select
	FD_ZERO (&readFdSet);
	FD_SET( m_fileDesc, &readFdSet);
	
	if ( timeOutus < 0) // pas de TIMEOUT
	{
		l_error = select(m_fileDesc+1, &readFdSet, 0, 0, NULL);
	}
	else
	{
		l_error = select(m_fileDesc+1, &readFdSet, 0, 0, &l_TimeOut);
	}
	
	if (l_error == 0)
	{
		return READ_TIMEDOUT;
	}
	else if (l_error < 0)
	{
		SerialException errMsg("Read error on serial device: ");
		errMsg << m_deviceName << ",  error code:" << l_error;
		throw(errMsg);
	}
	else
	{
		l_nbBytesRead = ::read(m_fileDesc, buf, nbBytesToRead);
		if (l_nbBytesRead == 0)
		{
			SerialException errMsg("EOF reached on serial device: ");
			errMsg << m_deviceName;
			throw( errMsg );
		}
		else if (l_nbBytesRead == -1)
		{
			SerialException errMsg("Read ERROR  on serial device: ");
			errMsg << m_deviceName << strerror( errno );
			throw( errMsg );
		}
		nbBytesToRead = l_nbBytesRead;
	}
	return READ_OK;
}


